"""
Tests for carta_envoy.launchers
"""
import logging
import os
from pathlib import Path
from unittest.mock import MagicMock, patch

# pylint: disable=E0401, R0201
import pytest

from carta_envoy.launchers import CartaLauncher, CartaWrapperLauncher

logger = logging.getLogger("casa_envoy")

UI_URL = "http://localhost:4444/workspaces"
CARTA_URL = UI_URL + "/carta/requests/-1/html"
test_settings = {
    "timeout": 1,
    "carta_path": "/fake/path/to/nowhere",
    "redis_host": "noplace.nrao.edu",
    "redis_port": "6397",
    "redis_password": "password",
    "reverse_proxy": "black-abyss.nrao.edu",
    "data_location": ".",
    "useCarta": False,
    "notification_url": "http://notification:3458",
    "workflow_url": "http://workflow:3456",
    "wf_request_id": -1,
}

launcher = CartaLauncher(settings=test_settings)

SUBPROCESS_COMMAND_PATCH = "carta_envoy.launchers.subprocess.Popen"

SESSION_PORT = 7777
WRAPPER_PORT = 1010


class TestCartaWrapperLauncher:
    # NOTE: Not testing run method as all it does is run a web server, which we don't want
    #  to do in a test

    def test_deploy_wrapper_html(self):
        path_to_wrapper = Path("index.html")
        assert not path_to_wrapper.exists()

        CartaWrapperLauncher.deploy_wrapper_html(Path(), "carta_url", "timeout_date")
        assert path_to_wrapper.exists()
        html_content = path_to_wrapper.read_text()
        assert all(keyword in html_content for keyword in ['"carta_url"', '"timeout_date"'])

        # Clean up wrapper
        os.remove("index.html")
        assert not path_to_wrapper.exists()


class TestCartaLauncher:
    """
    Exercise various CartaLauncher abilities.

    """

    def test_prepare_carta(self):
        """
        Make sure Redis is prepared for CARTA.

        :return:
        """
        with patch("carta_envoy.connect.RedisConnect.prepare_redis") as redis_prep:
            launcher.prepare_for_carta()
            assert redis_prep.call_count == 1

    def test_launch(self):
        """
        Confirm that CartaLauncher behaves as expected.

        :return:
        """
        with patch("carta_envoy.launchers.CartaLauncher.prepare_for_carta") as prep:
            with patch("carta_envoy.launchers.CartaLauncher.run_carta") as carta:
                launcher.launch()
                assert prep.call_count == 1
                assert carta.call_count == 1

    def test_dont_run_carta(self):
        """
        Test that CARTA doesn't run if it isn't set to
        """
        with patch(SUBPROCESS_COMMAND_PATCH) as mock_subprocess:
            launcher.run_carta(
                path_to_carta="fake_carta_path",
                timeout_minutes=1,
                file_browser_path=Path(),
                session_port=SESSION_PORT,
                wrapper_port=WRAPPER_PORT,
                carta_url="carta_url",
                wrapper_url="wrapper_url",
                session_timeout_date="fake_date",
            )
            assert mock_subprocess.call_count == 0

    def test_run_carta_with_aat(self):
        """
        Test that CARTA runs successfully with setup and teardown
        """
        launcher.settings["useCarta"] = True
        launcher.settings["send_ready"] = "false"
        mock_archive_connect = MagicMock()
        launcher.archive_connect = mock_archive_connect

        with patch(SUBPROCESS_COMMAND_PATCH) as mock_subprocess:
            with patch("carta_envoy.launchers.Thread") as fake_wrapper_thread:
                fake_wrapper_thread.is_alive.return_value = True
                launcher.run_carta(
                    path_to_carta="fake_carta_path",
                    timeout_minutes=1,
                    file_browser_path=Path(),
                    session_port=SESSION_PORT,
                    wrapper_port=WRAPPER_PORT,
                    carta_url="carta_url",
                    wrapper_url="wrapper_url",
                    session_timeout_date="fake_date",
                )
                assert mock_subprocess.call_count == 1
                mock_subprocess.assert_called_with(
                    [
                        "fake_carta_path",
                        "--no_browser",
                        "--debug_no_auth",
                        "--no_runtime_config",
                        f"--port={SESSION_PORT!s}",
                        "--top_level_folder=.",
                        ".",
                    ],
                    preexec_fn=None,
                    stdin=-1,
                    stdout=-1,
                    stderr=-1,
                )
                mock_archive_connect.send_carta_url_to_rh.assert_called_with("wrapper_url")
                assert fake_wrapper_thread.call_count == 1

        # Clean up wrapper
        os.remove("index.html")

    @pytest.mark.skip("Messaging mocks are inoperable.")
    def test_run_carta_with_ws(self):
        """
        Test that CARTA runs successfully with setup and teardown
        """
        launcher.settings["useCarta"] = True
        launcher.settings["send_ready"] = "true"
        mock_notification_connect = MagicMock()
        launcher.notification = mock_notification_connect

        with patch(SUBPROCESS_COMMAND_PATCH) as mock_subprocess:
            with patch("carta_envoy.launchers.Thread") as fake_wrapper_thread:
                fake_wrapper_thread.is_alive.return_value = True
                launcher.run_carta(
                    path_to_carta="fake_carta_path",
                    timeout_minutes=1,
                    file_browser_path=Path(),
                    session_port=SESSION_PORT,
                    wrapper_port=WRAPPER_PORT,
                    carta_url="carta_url",
                    wrapper_url="wrapper_url",
                    session_timeout_date="fake_date",
                )
                assert mock_subprocess.call_count == 1
                mock_subprocess.assert_called_with(
                    [
                        "fake_carta_path",
                        "--no_browser",
                        "--debug_no_auth",
                        "--no_runtime_config",
                        f"--port={SESSION_PORT!s}",
                        "--top_level_folder=.",
                        ".",
                    ],
                    preexec_fn=None,
                    stdin=-1,
                    stdout=-1,
                    stderr=-1,
                )
                mock_notification_connect.send_session_ready.assert_called_with("wrapper_url")
                assert fake_wrapper_thread.call_count == 1

        # Clean up wrapper
        os.remove("index.html")
