"""
Tests for aat_wrest.observation_wrester
"""

import argparse
from enum import Enum
from unittest.mock import patch, MagicMock

# pylint: disable=E0401, E0402
import json
import pytest
from aat_wrest.observation_wrester import WrestObservationMetadata


class Keys(Enum):
    """The things we're reporting"""

    PROJECT = "projectCode"
    OBS_TIME = "timeObserved"
    QUEUE_TIME = "timeInQueue"


PROJECT = Keys.PROJECT.value
OBS_TIME = Keys.OBS_TIME.value
QUEUE_TIME = Keys.QUEUE_TIME.value


_17A_109_SPL = "uid://evla/execblock/5c71ade0-d035-4fd5-a36f-0389e34db0e5"
_17A_109_ARGS = argparse.Namespace(spl=_17A_109_SPL)
_17A_109_EXPECTED = {
    PROJECT: "17A-109",
    OBS_TIME: "0 days, 0 hours, 4 minutes, 34 seconds",
    QUEUE_TIME: "1125 days, 11 hours, 43 minutes, 58 seconds",
}

args = argparse.Namespace(observation="17B-197.sb34663512.eb34806505.58108.78427738426")

result_obs = (
    '{"spl": "uid://evla/execblock/8fbfb54b-d141-42fe-b079-609339a69cba", '
    '"bands": "Ka", '
    '"array_config": "D", '
    '"obs_start_time": "1234567", '
    '"obs_end_time": "123598898" }'
)


def mock_wrester(args: argparse.Namespace) -> WrestObservationMetadata:
    """
    Pretend to get the information from the MDDB.

    :param args:
    :return:
    """
    with patch("psycopg2.connect") as conn:
        return WrestObservationMetadata(conn, args.observation)


wrester = mock_wrester(args)


class TestWrestObservationInfo:
    @patch("json.dumps", MagicMock(return_value=result_obs))
    def test_wrest_observation_info(self):
        wrester.conn.cursor.return_value.fetchall.return_value = [
            "uid://evla/execblock/8fbfb54b-d141-42fe-b079-609339a69cba",
            "Ka",
            "D",
            "1234567",
            "123598898",
        ]

        assert args.observation == "17B-197.sb34663512.eb34806505.58108.78427738426"
        metadata = wrester.wrest_observation_info()

        assert (
            metadata == '{"spl": "uid://evla/execblock/8fbfb54b-d141-42fe-b079-609339a69cba", '
            '"bands": "Ka", '
            '"array_config": "D", '
            '"obs_start_time": "1234567", '
            '"obs_end_time": "123598898" }'
        )

    @pytest.mark.skip("Dates are broken. Method superseded by wrest_observation_info")
    @patch("json.dumps", MagicMock(return_value=_17A_109_EXPECTED))
    def test_gets_expected_observation_info(self):
        """
        Does ObservationWrester wrest the expected data from the MDDB?

        :return:
        """
        wrester = mock_wrester(_17A_109_ARGS)
        assert wrester.spl == _17A_109_SPL
        actual = wrester.wrest_observation_time_info()
        assert actual == _17A_109_EXPECTED

    @pytest.mark.skip("... Dates are broken... Method superseded by wrest_observation_info")
    def test_handles_evla_exec_block(self):
        """
        Confirm we get what we expect when observing time is minimal.
        :return:
        """
        spl = "uid://evla/execblock/91c685b6-4527-44b1-9f91-3904e1125817"
        args = argparse.Namespace(spl=spl)
        expected = {
            PROJECT: "19A-440",
            OBS_TIME: "0 days, 0 hours, 0 minutes, 4 seconds",
            QUEUE_TIME: "627 days, 22 hours, 43 minutes, 6 seconds",
        }

        with patch("json.dumps", MagicMock(return_value=expected)):
            wrester = mock_wrester(args)
            actual = wrester.wrest_observation_time_info()
            assert actual[PROJECT] == expected[PROJECT]
            obs_parts = actual[OBS_TIME].split(", ")
            seconds_part = obs_parts[3].split(" ")
            assert int(seconds_part[0]) == 4
